require "import"
import "android.widget.*"
import "android.view.*"
import "android.speech.RecognizerIntent"
import "android.speech.SpeechRecognizer"
import "android.speech.RecognitionListener"
import "android.net.Uri"
import "android.content.Intent"
import "android.content.Context"
import "android.content.ClipData"
import "android.content.ClipboardManager"
import "java.util.Locale"

local spokenText = ""

-- UI Layout
local layout = {
  ScrollView,
  {
    LinearLayout,
    orientation = "vertical",
    padding = "16dp",
    gravity = "center",
    {
      TextView,
      id = "welcomeText",
      text = "hello, welcome to a smart speech to text converter.",
      textSize = "18sp",
      gravity = "center",
      layout_width = "match_parent",
      layout_height = "wrap_content"
    },
    {
      TextView,
      text = "this plugin is developed by Sujan Rai at Tech Visionary",
      textSize = "16sp",
      gravity = "center",
      layout_width = "match_parent",
      layout_height = "wrap_content"
    },
    {
      TextView,
      text = "join my telegram channel for more apps and resources",
      textSize = "16sp",
      gravity = "center",
      layout_width = "match_parent",
      layout_height = "wrap_content"
    },
    {
      Button,
      id = "telegramBtn",
      text = "Join Telegram Channel",
      layout_width = "wrap_content",
      layout_height = "wrap_content"
    },
    {
      Button,
      id = "startBtn",
      text = "Start Speak",
      layout_width = "wrap_content",
      layout_height = "wrap_content"
    },
    {
      TextView,
      id = "convertedText",
      text = "Recognized Text Will Appear Here...",
      textSize = "16sp",
      padding = "10dp",
      layout_width = "match_parent",
      layout_height = "wrap_content",
      background = "#EEEEEE"
    },
    {
      LinearLayout,
      orientation = "horizontal",
      gravity = "center",
      {
        Button,
        id = "copyBtn",
        text = "Copy Text",
        layout_width = "wrap_content",
        layout_height = "wrap_content"
      },
      {
        Button,
        id = "exitBtn",
        text = "Exit",
        layout_width = "wrap_content",
        layout_height = "wrap_content"
      }
    }
  }
}

activity.setContentView(loadlayout(layout))

-- Telegram button
telegramBtn.onClick = function()
  local url = "https://t.me/techvisionarynepal"
  local intent = Intent(Intent.ACTION_VIEW)
  intent.setData(Uri.parse(url))
  activity.startActivity(intent)
end

-- Exit button
exitBtn.onClick = function()
  activity.finish()
end

-- Copy text
copyBtn.onClick = function()
  local clipboard = activity.getSystemService(Context.CLIPBOARD_SERVICE)
  local clip = ClipData.newPlainText("voiceText", convertedText.getText())
  clipboard.setPrimaryClip(clip)
  Toast.makeText(activity, "Text copied to clipboard", Toast.LENGTH_SHORT).show()
end

-- Save recognized text
function saveWords(words)
  spokenText = words
  convertedText.setText(words)
end

-- Start speech recognition
function startListening()
  pcall(function()
    local recognizerIntent = Intent(RecognizerIntent.ACTION_RECOGNIZE_SPEECH)
    recognizerIntent.putExtra(RecognizerIntent.EXTRA_LANGUAGE_MODEL, RecognizerIntent.LANGUAGE_MODEL_FREE_FORM)
    recognizerIntent.putExtra(RecognizerIntent.EXTRA_LANGUAGE, "ne-NP")
    recognizerIntent.putExtra(RecognizerIntent.EXTRA_CALLING_PACKAGE, activity.getPackageName())

    local speechRecognizer = SpeechRecognizer.createSpeechRecognizer(activity)

    local speechListener = RecognitionListener {
      onResults = function(results)
        local data = results.getParcelableArrayList(SpeechRecognizer.RESULTS_RECOGNITION)
        if data ~= nil and data.size() > 0 then
          local recognizedText = data.get(0)
          saveWords(recognizedText)
        else
          convertedText.setText("No speech recognized.")
        end
      end,
      onError = function(errorCode)
        local errorMessages = {
          [1] = "Network timeout",
          [2] = "Network error",
          [3] = "Audio error",
          [4] = "Server error",
          [5] = "Client error",
          [6] = "Speech timeout",
          [7] = "No match found",
          [8] = "Recognizer busy",
          [9] = "Insufficient permissions"
        }
        local message = errorMessages[errorCode] or "Unknown error"
        convertedText.setText("Recognition error: " .. message)
        speechRecognizer.destroy()
      end
    }

    speechRecognizer.setRecognitionListener(speechListener)
    speechRecognizer.startListening(recognizerIntent)
  end)
end

-- Start button
startBtn.onClick = function()
  startListening()
end
